
static char help[] = "Matrix Partitioning Without Spectral Graph Partitioning By Using ParMETIS and CHACO.\n\n";
  
#include <petscmat.h>   

#include <petscis.h> 

#include <petscsys.h>

#include <stdio.h>

#include <stdlib.h>   

#include <petsctime.h>


PetscErrorCode absmat(Mat A,Mat *AbsA)                                              
{
  PetscErrorCode ierr;
  PetscInt       n,i,nc,j,rstart,rend;
  const PetscInt *aj;                                                                
  const PetscScalar *aa;
  PetscScalar *absaa; 
  
  ierr = MatGetSize(A,&n,NULL);CHKERRQ(ierr);
  ierr = MatDuplicate(A,MAT_COPY_VALUES,AbsA);CHKERRQ(ierr);
       
  
  /* copy over the matrix entries from the matrix A to matrix AbsA*/
  ierr = MatGetOwnershipRange(A,&rstart,&rend);CHKERRQ(ierr);                  
  

  for (i=rstart; i<rend; i++) {                                                 
    ierr = MatGetRow(A,i,&nc,&aj,&aa);CHKERRQ(ierr);                               
    
    /* replace the nonzero values with their absolute values*/
    
    PetscMalloc1(nc,&absaa);
    for (j=0; j<nc; j++){
        absaa[j] = fabs(aa[j]);
    }
    
    ierr = MatSetValues(*AbsA,1,&i,nc,aj,absaa,INSERT_VALUES);CHKERRQ(ierr);    
    ierr = MatRestoreRow(A,i,&nc,&aj,&aa);CHKERRQ(ierr);
  }
                    
  return(0);          
}


int main (int argc,char **argv)                                                     
{
                      
  Mat               A=NULL,AL;                                                       
  IS                partitioning;
  PetscViewer       fd; 
  char              file[PETSC_MAX_PATH_LEN];
  PetscBool         flg;  
  PetscErrorCode    ierr;
  MatPartitioning   part;                                       

  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;

  ierr = PetscOptionsGetString(NULL,NULL,"-f",file,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
  if (!flg) SETERRQ(PETSC_COMM_WORLD,1,"Must indicate binary file with the -f option");

  ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file,FILE_MODE_READ,&fd);CHKERRQ(ierr);

  ierr = MatCreate(PETSC_COMM_WORLD,&A);CHKERRQ(ierr);  
    MatSetType(A,MATMPIAIJ); 
  ierr = MatSetOptionsPrefix(A,"a_");CHKERRQ(ierr);
  ierr = MatSetFromOptions(A);CHKERRQ(ierr);
  ierr = MatLoad(A,fd);CHKERRQ(ierr);
  ierr = PetscViewerDestroy(&fd);CHKERRQ(ierr);
  
   /* Symmetry check */
      
      Mat Atr,SymmA,Atrabs,Aabs;
      PetscBool isEqual;
      PetscInt i;

    /*ierr = MatTranspose(A,MAT_INITIAL_MATRIX,&Atr);CHKERRQ(ierr);
    ierr = MatDuplicate(Atr,MAT_COPY_VALUES,&SymmA);CHKERRQ(ierr);*/
      flg = PETSC_TRUE;                                                           
    PetscOptionsGetBool(NULL,NULL, "-check_symmetry", &flg,NULL);
    if (flg) {
        MatIsSymmetric(A,0.0,&isEqual);
        if (isEqual) {
        ierr = absmat(A,&SymmA);CHKERRQ(ierr);

        } else {
            ierr = MatTranspose(A,MAT_INITIAL_MATRIX,&Atr);CHKERRQ(ierr);
                ierr = absmat(Atr,&Atrabs);CHKERRQ(ierr);
                ierr = absmat(A,&Aabs);CHKERRQ(ierr);
                ierr = MatAssemblyBegin(Atrabs,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);      
                ierr = MatAssemblyEnd(Atrabs,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
                ierr = MatDuplicate(Atrabs,MAT_COPY_VALUES,&SymmA);CHKERRQ(ierr);
                ierr = MatAssemblyBegin(Aabs,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);      
                ierr = MatAssemblyEnd(Aabs,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
                ierr = MatAXPY(SymmA,1.,Aabs,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr); 
        }

    }
        ierr = MatAssemblyBegin(SymmA,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
        ierr = MatAssemblyEnd(SymmA,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
        

  ierr = MatConvert(SymmA,MATMPIADJ,MAT_INITIAL_MATRIX,&AL);CHKERRQ(ierr);                           

  ierr = MatPartitioningCreate(MPI_COMM_WORLD,&part);CHKERRQ(ierr);
  ierr = MatPartitioningSetAdjacency(part,AL);CHKERRQ(ierr); 
  
  ierr = MatPartitioningSetFromOptions(part);CHKERRQ(ierr);
  ierr = MatPartitioningApply(part,&partitioning);CHKERRQ(ierr);

/*
  To find the approx cut, we first need to apply idU to our original matrix since this index set results from symmetric approx SymmA. 
*/

 PetscMPIInt rank,size;
  MPI_Comm_rank(PETSC_COMM_WORLD, &rank);
  MPI_Comm_size(PETSC_COMM_WORLD, &size);

PetscInt siz,sizm,j,k;
MatGetSize(A,&siz,&sizm);

//Get all the index set to one piece
IS isall;
ISAllGather(partitioning,&isall);
const PetscInt *idUarr;
 ISGetIndices(isall,&idUarr);
 
PetscScalar *idUarrint;
PetscMalloc1(siz,&idUarrint);

for (i=0; i<siz; i++){
    idUarrint[i] = idUarr[i];
}

PetscInt *idxU,idm,idn;
ierr = PetscMalloc1(siz,&idxU);CHKERRQ(ierr);
  for (i=0; i<siz;i++){
      idxU[i] = i;
  }

PetscSortRealWithPermutation(siz,idUarrint,idxU);
MatGetLocalSize(A,&idm,&idn);

PetscInt sizofidxU;
PetscInt *idxUsub;
PetscMalloc1(siz,&idxUsub);                                                     

for(i=0;i<siz;i++){
    if (i != 0){
        if (idUarrint[idxU[i]]== idUarrint[idxU[i-1]]){
            continue;
        }
    }
    if (idUarrint[idxU[i]]== rank){
        sizofidxU = 0;
        k = i;
        j = 0;
        while (idUarrint[idxU[k]] == rank){
            idxUsub[j] = idxU[k];
            sizofidxU++;
            k++;
            j++;
            if (k == siz){
                break;
            }
        }
    }
}

IS idUi;
ISCreateGeneral(PETSC_COMM_WORLD,sizofidxU,idxUsub,PETSC_COPY_VALUES,&idUi);           


/*
  To find the approx cut, we first need to apply idU to our original matrix since this index set results from symmetric approx SymmA. 
*/

  /*Permute matrix L (spy(A(idu,idU)))*/

  Mat Ais,PL;
  MatCreateSubMatrix(A,idUi,idUi,MAT_INITIAL_MATRIX,&Ais);  
  ISSetPermutation(idUi);
  MatPermute(Ais,idUi,idUi,&PL);

  
  ierr = MatDestroy(&A);CHKERRQ(ierr);                    
  
  ierr = PetscFinalize();

  return ierr;
} 


/*TEST

   build:
      requires:  !complex !define(PETSC_USE_64BIT_INDICES), parmetis

   test:
      args: -f Stranke94_binary  -mat_partitioning_type chaco
      
      example: mpirun -n 1 ./sample -f Stranke94_binary -mat_partitioning_type chaco 


TEST*/

